@@ -22,6 +22,7 @@
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
 #include <libwnck/libwnck.h>
 #include <string.h>
@@ -1118,6 +1119,10 @@ void stickynotes_remove(StickyNote *note)
 void
 stickynotes_save_now (StickyNotesApplet *applet)
 {
+	char *path;
+	char *notes_file;
+	xmlDocPtr doc;
+	xmlNodePtr root;
 	WnckScreen *wnck_screen;
 	const gchar *title;
 	GtkTextBuffer *buffer;
@@ -1125,9 +1130,26 @@ stickynotes_save_now (StickyNotesApplet *applet)
 	gchar *body;
 	guint i;
 
+	path = g_build_filename (g_get_user_config_dir (),
+	                         "gnome-applets",
+	                         "sticky-notes",
+	                         NULL);
+
+	g_mkdir_with_parents (path, 0700);
+	notes_file = g_build_filename (path, applet->filename, NULL);
+	g_free (path);
+
+	if (applet->notes == NULL) {
+		g_unlink (notes_file);
+		g_free (notes_file);
+
+		applet->save_scheduled = FALSE;
+		return;
+	}
+
 	/* Create a new XML document */
-	xmlDocPtr doc = xmlNewDoc(XML_CHAR ("1.0"));
-	xmlNodePtr root = xmlNewDocNode(doc, NULL, XML_CHAR ("stickynotes"), NULL);
+	doc = xmlNewDoc(XML_CHAR ("1.0"));
+	root = xmlNewDocNode(doc, NULL, XML_CHAR ("stickynotes"), NULL);
 
 	xmlDocSetRootElement(doc, root);
 	xmlNewProp(root, XML_CHAR("version"), XML_CHAR (VERSION));
@@ -1211,21 +1233,19 @@ stickynotes_save_now (StickyNotesApplet *applet)
 	}
 
 	{
-		const gchar *dir;
-		gchar *path;
-		gchar *file;
+		char *tmp_file;
 
-		dir = g_get_user_config_dir ();
-		path = g_build_filename (dir, "gnome-applets", "sticky-notes", NULL);
-		file = g_build_filename (path, "sticky-notes.xml", NULL);
+		tmp_file = g_strdup_printf ("%s.tmp", notes_file);
+		if (xmlSaveFormatFile (tmp_file, doc, 1) == -1 ||
+			g_rename (tmp_file, notes_file) == -1) {
+			g_warning ("Failed to save notes");
+			g_unlink (tmp_file);
+		}
 
-		g_mkdir_with_parents (path, 0700);
-		g_free (path);
-
-		xmlSaveFormatFile (file, doc, 1);
-		g_free (file);
+		g_free (tmp_file);
 	}
 
+	g_free (notes_file);
 	xmlFreeDoc(doc);
 
 	applet->save_scheduled = FALSE;
@@ -1253,44 +1273,153 @@ stickynotes_save (StickyNotesApplet *applet)
   }
 }
 
+static char *
+get_unique_filename (void)
+{
+  int number;
+  char *filename;
+  char *path;
+
+  number = 1;
+  filename = NULL;
+  path = NULL;
+
+  do
+    {
+      g_free (filename);
+      filename = g_strdup_printf ("sticky-notes-%d.xml", number++);
+
+      g_free (path);
+      path = g_build_filename (g_get_user_config_dir (),
+                               "gnome-applets",
+                               "sticky-notes",
+                               filename,
+                               NULL);
+    }
+  while (g_file_test (path, G_FILE_TEST_EXISTS));
+
+  g_free (path);
+
+  return filename;
+}
+
+static char *
+get_notes_file (StickyNotesApplet *applet)
+{
+  const char *dir;
+  char *filename;
+  char *notes_file;
+
+  dir = g_get_user_config_dir ();
+  filename = g_settings_get_string (applet->settings, KEY_FILENAME);
+
+  g_free (applet->filename);
+
+  if (*filename == '\0')
+    {
+      g_free (filename);
+      filename = get_unique_filename ();
+
+      notes_file = g_build_filename (dir,
+                                     "gnome-applets",
+                                     "sticky-notes",
+                                     filename,
+                                     NULL);
+
+
+
+      applet->filename = filename;
+      g_settings_set_string (applet->settings, KEY_FILENAME, filename);
+    }
+  else
+    {
+      applet->filename = filename;
+      notes_file = g_build_filename (dir,
+                                     "gnome-applets",
+                                     "sticky-notes",
+                                     filename,
+                                     NULL);
+    }
+
+  return notes_file;
+}
+
+/**
+ * The sticky-notes file had various locations in past versions of the applet.
+ *
+ * Until version 3.24 the location was: `~/.config/gnome-applets/stickynotes`.
+ * In version 3.26 the location changed to: `<XDG_CONFIG_HOME>/gnome-applets/sticky-notes/sticky-notes.xml`.
+ * In version 3.38 the location changed to: `<XDG_CONFIG_HOME>/gnome-applets/sticky-notes/sticky-notes-%d.xml`
+ * where `%d` is incremented for each notes applet on the panel.
+ */
+static void
+migrate_legacy_note_file (StickyNotesApplet *applet,
+                          const char        *notes_file)
+{
+  const char *dir;
+  char *old_file;
+
+  dir = g_get_user_config_dir ();
+
+  old_file = g_build_filename (dir,
+                               "gnome-applets",
+                               "sticky-notes",
+                               "sticky-notes.xml",
+                               NULL);
+
+  if (g_file_test (old_file, G_FILE_TEST_EXISTS))
+    {
+      g_rename (old_file, notes_file);
+
+      g_free (old_file);
+      old_file = g_build_filename (dir,
+                                   "gnome-applets",
+                                   "stickynotes",
+                                   NULL);
+
+      if (g_file_test (old_file, G_FILE_TEST_EXISTS))
+        g_unlink (old_file);
+    }
+  else
+    {
+      g_free (old_file);
+      old_file = g_build_filename (dir,
+                                   "gnome-applets",
+                                   "stickynotes",
+                                   NULL);
+
+      if (g_file_test (old_file, G_FILE_TEST_EXISTS))
+        g_rename (old_file, notes_file);
+    }
+
+  g_free (old_file);
+}
+
 /* Load all sticky notes from an XML configuration file */
 void
 stickynotes_load (StickyNotesApplet *applet)
 {
+	char *notes_file;
 	xmlDocPtr doc;
 	xmlNodePtr root;
 	xmlNodePtr node;
-	/* WnckScreen *wnck_screen; */
+
 	GList *new_notes, *tmp1;  /* Lists of StickyNote*'s */
-	GList *new_nodes; /* Lists of xmlNodePtr's */
 	int x, y, w, h;
 
-	{
-		const gchar *dir;
-		gchar *file;
+	notes_file = get_notes_file (applet);
 
-		dir = g_get_user_config_dir ();
-		file = g_build_filename (dir, "gnome-applets", "sticky-notes",
-		                         "sticky-notes.xml", NULL);
+	if (!g_file_test (notes_file, G_FILE_TEST_EXISTS))
+		migrate_legacy_note_file (applet, notes_file);
 
-		if (!g_file_test (file, G_FILE_TEST_EXISTS))
-		{
-			g_free (file);
-			file = g_build_filename (dir, "gnome-applets",
-			                         "stickynotes", NULL);
-		}
-
-		doc = xmlParseFile (file);
-		g_free (file);
-	}
-
-	/* If the XML file does not exist, create a blank one */
-	if (!doc)
-	{
-		stickynotes_save (applet);
+	if (!g_file_test (notes_file, G_FILE_TEST_EXISTS)) {
+		g_free (notes_file);
 		return;
 	}
 
+	doc = xmlParseFile (notes_file);
+	g_free (notes_file);
+
 	/* If the XML file is corrupted/incorrect, create a blank one */
 	root = xmlDocGetRootElement(doc);
 	if (!root || xmlStrcmp(root->name, XML_CHAR ("stickynotes")))
@@ -1304,7 +1433,7 @@ stickynotes_load (StickyNotesApplet *applet)
 
 	/* For all children of the root node (ie all sticky notes) */
 	new_notes = NULL;
-	new_nodes = NULL;
+
 	while (node) {
 		if (!xmlStrcmp(node->name, (const xmlChar *) "note"))
 		{
@@ -1353,7 +1482,6 @@ stickynotes_load (StickyNotesApplet *applet)
 			note = stickynote_new_aux (applet, x, y, w, h);
 			applet->notes = g_list_append (applet->notes, note);
 			new_notes = g_list_append (new_notes, note);
-			new_nodes = g_list_append (new_nodes, node);
 
 			/* Retrieve and set title of the note */
 			{
@@ -1435,10 +1563,6 @@ stickynotes_load (StickyNotesApplet *applet)
 	}
 
 	tmp1 = new_notes;
-	/*
-	wnck_screen = wnck_screen_get_default ();
-	wnck_screen_force_update (wnck_screen);
-	*/
 
 	while (tmp1)
 	{
@@ -1449,7 +1573,6 @@ stickynotes_load (StickyNotesApplet *applet)
 	}
 
 	g_list_free (new_notes);
-	g_list_free (new_nodes);
 
 	xmlFreeDoc(doc);
 }
